from typing import Any

from sqlalchemy.dialects.firebird.base import BIGINT as BIGINT
from sqlalchemy.dialects.firebird.base import BLOB as BLOB
from sqlalchemy.dialects.firebird.base import CHAR as CHAR
from sqlalchemy.dialects.firebird.base import DATE as DATE
from sqlalchemy.dialects.firebird.base import FLOAT as FLOAT
from sqlalchemy.dialects.firebird.base import NUMERIC as NUMERIC
from sqlalchemy.dialects.firebird.base import SMALLINT as SMALLINT
from sqlalchemy.dialects.firebird.base import TEXT as TEXT
from sqlalchemy.dialects.firebird.base import TIME as TIME
from sqlalchemy.dialects.firebird.base import TIMESTAMP as TIMESTAMP
from sqlalchemy.dialects.firebird.base import VARCHAR as VARCHAR

dialect: Any
