from typing import Any
from typing import Optional

from ..ext.hybrid import hybrid_property

class index_property(hybrid_property):
    attr_name: Any = ...
    index: Any = ...
    default: Any = ...
    datatype: Any = ...
    onebased: Any = ...
    def __init__(
        self,
        attr_name: Any,
        index: Any,
        default: Any = ...,
        datatype: Optional[Any] = ...,
        mutable: bool = ...,
        onebased: bool = ...,
    ): ...
    def fget(self, instance: Any): ...
    def fset(self, instance: Any, value: Any) -> None: ...
    def fdel(self, instance: Any) -> None: ...
    def expr(self, model: Any): ...
