from typing import Any

from .schema import Table

class ConventionDict:
    const: Any = ...
    table: Any = ...
    convention: Any = ...
    def __init__(self, const: Any, table: Table, convention: Any) -> None: ...
    def __getitem__(self, key: Any) -> Any: ...
