from typing import Any
from typing import Callable
from typing import Set
from typing import TypeVar

_F = TypeVar("_F", bound=Callable[..., Any])

class _ModuleRegistry:
    module_registry: Set[str] = ...
    prefix: str = ...
    def __init__(self, prefix: str = ...) -> None: ...
    def preload_module(self, *deps: str) -> Callable[[_F], _F]: ...
    def import_prefix(self, path: Any) -> None: ...

preloaded: _ModuleRegistry

def preload_module(*deps: str) -> Callable[[_F], _F]: ...
