from typing import Any

from .base import BFILE as BFILE
from .base import BINARY_DOUBLE as BINARY_DOUBLE
from .base import BINARY_FLOAT as BINARY_FLOAT
from .base import BLOB as BLOB
from .base import CHAR as CHAR
from .base import CLOB as CLOB
from .base import DATE as DATE
from .base import DOUBLE_PRECISION as DOUBLE_PRECISION
from .base import FLOAT as FLOAT
from .base import INTERVAL as INTERVAL
from .base import LONG as LONG
from .base import NCHAR as NCHAR
from .base import NCLOB as NCLOB
from .base import NUMBER as NUMBER
from .base import NVARCHAR as NVARCHAR
from .base import NVARCHAR2 as NVARCHAR2
from .base import RAW as RAW
from .base import ROWID as ROWID
from .base import TIMESTAMP as TIMESTAMP
from .base import VARCHAR as VARCHAR
from .base import VARCHAR2 as VARCHAR2

dialect: Any
