from typing import Any

from .pysqlite import SQLiteDialect_pysqlite as SQLiteDialect_pysqlite
from ... import pool as pool

class SQLiteDialect_pysqlcipher(SQLiteDialect_pysqlite):
    driver: str = ...
    pragmas: Any = ...
    @classmethod
    def dbapi(cls): ...
    @classmethod
    def get_pool_class(cls, url: Any): ...
    def connect(self, *cargs: Any, **cparams: Any): ...
    def create_connect_args(self, url: Any): ...

dialect = SQLiteDialect_pysqlcipher
