from typing import Any

from .base import BIGINT as BIGINT
from .base import BINARY as BINARY
from .base import BIT as BIT
from .base import BLOB as BLOB
from .base import BOOLEAN as BOOLEAN
from .base import CHAR as CHAR
from .base import DATE as DATE
from .base import DATETIME as DATETIME
from .base import DECIMAL as DECIMAL
from .base import DOUBLE as DOUBLE
from .base import ENUM as ENUM
from .base import FLOAT as FLOAT
from .base import INTEGER as INTEGER
from .base import JSON as JSON
from .base import LONGBLOB as LONGBLOB
from .base import LONGTEXT as LONGTEXT
from .base import MEDIUMBLOB as MEDIUMBLOB
from .base import MEDIUMINT as MEDIUMINT
from .base import MEDIUMTEXT as MEDIUMTEXT
from .base import NCHAR as NCHAR
from .base import NUMERIC as NUMERIC
from .base import NVARCHAR as NVARCHAR
from .base import REAL as REAL
from .base import SET as SET
from .base import SMALLINT as SMALLINT
from .base import TEXT as TEXT
from .base import TIME as TIME
from .base import TIMESTAMP as TIMESTAMP
from .base import TINYBLOB as TINYBLOB
from .base import TINYINT as TINYINT
from .base import TINYTEXT as TINYTEXT
from .base import VARBINARY as VARBINARY
from .base import VARCHAR as VARCHAR
from .base import YEAR as YEAR
from .dml import Insert as Insert
from .dml import insert as insert

dialect: Any
