from typing import Any
from typing import Optional

from .mysqldb import MySQLDialect_mysqldb as MySQLDialect_mysqldb
from ...util import langhelpers as langhelpers
from ...util import py3k as py3k

class MySQLDialect_pymysql(MySQLDialect_mysqldb):
    driver: str = ...
    description_encoding: Any = ...
    supports_unicode_statements: bool = ...
    supports_unicode_binds: bool = ...
    def supports_server_side_cursors(self): ...
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(
        self, url: Any, _translate_args: Optional[Any] = ...
    ): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = MySQLDialect_pymysql
