from typing import Any
from typing import Optional

class scoped_session:
    session_factory: Any = ...
    registry: Any = ...
    def __init__(
        self, session_factory: Any, scopefunc: Optional[Any] = ...
    ) -> None: ...
    def __call__(self, **kw: Any): ...
    def remove(self) -> None: ...
    def configure(self, **kwargs: Any) -> None: ...
    def query_property(self, query_cls: Optional[Any] = ...): ...

ScopedSession = scoped_session
