from typing import Any

from .. import event

class DDLEvents(event.Events):
    def before_create(
        self, target: Any, connection: Any, **kw: Any
    ) -> None: ...
    def after_create(
        self, target: Any, connection: Any, **kw: Any
    ) -> None: ...
    def before_drop(self, target: Any, connection: Any, **kw: Any) -> None: ...
    def after_drop(self, target: Any, connection: Any, **kw: Any) -> None: ...
    def before_parent_attach(self, target: Any, parent: Any) -> None: ...
    def after_parent_attach(self, target: Any, parent: Any) -> None: ...
    def column_reflect(
        self, inspector: Any, table: Any, column_info: Any
    ) -> None: ...
