from typing import Any

from .base import DATE as DATE
from .base import DATETIME as DATETIME
from .base import SQLiteDialect as SQLiteDialect
from ... import exc as exc
from ... import pool as pool
from ... import util as util

class _SQLite_pysqliteTimeStamp(DATETIME):
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class _SQLite_pysqliteDate(DATE):
    def bind_processor(self, dialect: Any): ...
    def result_processor(self, dialect: Any, coltype: Any): ...

class SQLiteDialect_pysqlite(SQLiteDialect):
    default_paramstyle: str = ...
    colspecs: Any = ...
    description_encoding: Any = ...
    driver: str = ...
    @classmethod
    def dbapi(cls): ...
    @classmethod
    def get_pool_class(cls, url: Any): ...
    def set_isolation_level(self, connection: Any, level: Any): ...
    def on_connect(self): ...
    def create_connect_args(self, url: Any): ...
    def is_disconnect(self, e: Any, connection: Any, cursor: Any): ...

dialect = SQLiteDialect_pysqlite
