from typing import Any

from .. import exc as exc
from .. import util as util
from ..sql import sqltypes as sqltypes

def compiles(class_: Any, *specs: Any): ...
def deregister(class_: Any) -> None: ...

class _dispatcher:
    specs: Any = ...
    def __init__(self) -> None: ...
    def __call__(self, element: Any, compiler: Any, **kw: Any): ...
